@@ -28,7 +28,9 @@
 #include <signal.h>
 #include <errno.h>
 #include <dirent.h>
+#ifdef __GLIBC__
 #include <execinfo.h>
+#endif
 #include <ctype.h>
 #include <sys/wait.h>
 #include <endian.h>
@@ -38,6 +40,8 @@
 char *add_slot_fname = ADD_SLOT_FNAME;
 char *remove_slot_fname = REMOVE_SLOT_FNAME;
 
+#define DR_MAX_LOG_SZ (1 << 20)
+
 #define DR_LOG_PATH	"/var/log/drmgr"
 #define DR_LOG_PATH0	"/var/log/drmgr.0"
 
@@ -214,7 +218,7 @@ inline void
 dr_fini(void)
 {
 	struct stat sbuf;
-	int max_dr_log_sz = 25000;
+	int max_dr_log_sz = DR_MAX_LOG_SZ;
 	int rc;
 	time_t t;
 	char tbuf[128];
@@ -853,6 +857,7 @@ sighandler(int signo)
 	say(ERROR, "Received signal %d, attempting to cleanup and exit\n",
 	    signo);
 
+#ifdef __GLIBC__
 	if (log_fd) {
 		void *callstack[128];
 		int sz;
@@ -860,6 +865,7 @@ sighandler(int signo)
 		sz = backtrace(callstack, 128);
 		backtrace_symbols_fd(callstack, sz, log_fd);
 	}
+#endif
 
 	dr_fini();
 	exit(-1);
@@ -925,8 +931,10 @@ sig_setup(void)
 	if (sigaction(SIGBUS, &sigact, NULL))
 		return -1;
 
+#ifdef __GLIBC__
 	/* dummy call to backtrace to get symbol loaded */
 	backtrace(callstack, 128);
+#endif
 	return 0;
 }
 
@@ -976,6 +984,15 @@ char *php_slot_type_msg[]={
 	"PCI-E capable, Rev 4, 16x lanes with 16x lanes connected",
 	"U.2 PCI-E capable, Rev 3, 4x lanes with 4x lanes connected",
 	"U.2 PCI-E capable, Rev 4, 4x lanes with 4x lanes connected",
+	"U.2 PCI-E capable, Rev 4, 4x lanes with 2x lanes connected", 	/* 45 */
+	"PCI-E capable, Rev 5, 8x lanes with 1 lane connected",
+	"PCI-E capable, Rev 5, 8x lanes with 4x lanes connected",
+	"PCI-E capable, Rev 5, 8x lanes with 8x lanes connected",
+	"PCI-E capable, Rev 5, 16x lanes with 1 lane connected",
+	"PCI-E capable, Rev 5, 16x lanes with 4x lanes connected",	/* 50 */
+	"PCI-E capable, Rev 5, 16x lanes with 8x lanes connected",
+	"U.2 PCI-E capable, Rev 5, 4x lanes with 2x lanes connected",
+	"U.2 PCI-E capable, Rev 5, 4x lanes with 4x lanes connected",
 };
 
 char *
@@ -986,7 +1003,7 @@ node_type(struct dr_node *node)
 
 	desc_msg_num = atoi(node->drc_type);
 	if ((desc_msg_num >= 1 &&  desc_msg_num <= 8) ||
-	    (desc_msg_num >= 11 && desc_msg_num <= 44))
+	    (desc_msg_num >= 11 && desc_msg_num <= 53))
 		desc = php_slot_type_msg[desc_msg_num];
 	else {
 		switch (node->dev_type) {
@@ -1462,39 +1479,43 @@ int kernel_dlpar_exists(void)
 }
 
 /**
- * do_kernel_dlpar
+ * do_kernel_dlpar_common
  * @brief Use the in-kernel dlpar capabilities to perform the requested
  *        dlpar operation.
  *
  * @param cmd command string to write to sysfs
+ * @silent_error if not 0, error is not reported, it's up to the caller
  * @returns 0 on success, !0 otherwise
  */
-int do_kernel_dlpar(const char *cmd, int cmdlen)
+int do_kernel_dlpar_common(const char *cmd, int cmdlen, int silent_error)
 {
-	int fd, rc;
-	int my_errno;
+	static int fd = -1;
+	int rc;
 
 	say(DEBUG, "Initiating kernel DLPAR \"%s\"\n", cmd);
 
 	/* write to file */
-	fd = open(SYSFS_DLPAR_FILE, O_WRONLY);
-	if (fd <= 0) {
-		say(ERROR, "Could not open %s to initiate DLPAR request\n",
-		    SYSFS_DLPAR_FILE);
-		return -1;
+	if (fd == -1) {
+		fd = open(SYSFS_DLPAR_FILE, O_WRONLY);
+		if (fd < 0) {
+			say(ERROR,
+			    "Could not open %s to initiate DLPAR request\n",
+			    SYSFS_DLPAR_FILE);
+			return -1;
+		}
 	}
 
 	rc = write(fd, cmd, cmdlen);
-	my_errno = errno;
-	close(fd);
 	if (rc <= 0) {
+		if (silent_error)
+			return (errno == 0) ? -1 : -errno;
 		/* write does not set errno for rc == 0 */
 		say(ERROR, "Failed to write to %s: %s\n", SYSFS_DLPAR_FILE,
-		    (rc == 0) ? "wrote 0 bytes" : strerror(my_errno));
+		    (rc == 0) ? "wrote 0 bytes" : strerror(errno));
 		return -1;
 	}
 
-	say(INFO, "Success\n");
+	say(DEBUG, "Success\n");
 	return 0;
 }
 
@@ -1524,6 +1545,12 @@ enum drc_type to_drc_type(const char *arg)
 	if (!strncmp(arg, "pmig", 4))
 		return DRC_TYPE_MIGRATION;
 
+	/*
+	 * Accelerator
+	 */
+	if (!strncmp(arg, "acc", 3))
+		return DRC_TYPE_ACC;
+
 	return DRC_TYPE_NONE;
 }
 
