@@ -27,6 +27,7 @@
 #include <string.h>
 #include <getopt.h>
 #include <unistd.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <sched.h>
 #include <signal.h>
@@ -42,6 +43,7 @@
 
 static bool o_legacy = false;
 static bool o_scaled = false;
+static bool o_security = false;
 
 static int threads_per_cpu;
 static int cpus_in_system;
@@ -163,7 +165,8 @@ static int assign_cpu_sysfs_fds(int threads_in_system)
 
 	return 0;
 error:
-	fprintf(stderr, "Failed to open %s\n", sysfs_file_path);
+	fprintf(stderr, "Failed to open %s: %s\n",
+			sysfs_file_path, strerror(errno));
 	close_cpu_sysfs_fds(threads_in_system);
 	return -1;
 }
@@ -180,7 +183,7 @@ int parse_sysfs_values(void)
 	for (i = 0; cpu_sysfs_fds[i].spurr > 0; i++) {
 		rc = pread(cpu_sysfs_fds[i].spurr, (void *)line, sizeof(line), 0);
 		if (rc == -1) {
-			fprintf(stderr, "Failed to /sys/devices/system/cpu/cpu%d/spurr\n",
+			fprintf(stderr, "Failed to read /sys/devices/system/cpu/cpu%d/spurr\n",
 					cpu_sysfs_fds[i].cpu);
 			goto check_cpu_hotplug;
 		}
@@ -190,7 +193,7 @@ int parse_sysfs_values(void)
 
 		rc = pread(cpu_sysfs_fds[i].idle_purr, (void *)line, sizeof(line), 0);
 		if (rc == -1) {
-			fprintf(stderr, "Failed to /sys/devices/system/cpu/cpu%d/idle_purr\n",
+			fprintf(stderr, "Failed to read /sys/devices/system/cpu/cpu%d/idle_purr\n",
 					cpu_sysfs_fds[i].cpu);
 			goto check_cpu_hotplug;
 		}
@@ -200,7 +203,7 @@ int parse_sysfs_values(void)
 
 		rc = pread(cpu_sysfs_fds[i].idle_spurr, (void *)line, sizeof(line), 0);
 		if (rc == -1) {
-			fprintf(stderr, "Failed to /sys/devices/system/cpu/cpu%d/idle_spurr\n",
+			fprintf(stderr, "Failed to read /sys/devices/system/cpu/cpu%d/idle_spurr\n",
 					cpu_sysfs_fds[i].cpu);
 			goto check_cpu_hotplug;
 		}
@@ -794,6 +797,10 @@ void get_node_name(struct sysentry *se, char *buf)
 
 void get_partition_name(struct sysentry *se, char *buf)
 {
+	if (se->value[0] != '\0') {
+		strcpy(buf, se->value);
+		return;
+	}
 	return get_name("/proc/device-tree/ibm,partition-name", buf);
 }
 
@@ -833,7 +840,7 @@ void get_mem_total(struct sysentry *se, char *buf)
 	*nl = '\0';
 
 	if (o_legacy) {
-		sprintf(buf, "%d %s", atoi(mem) / 1024, "MB");
+		sprintf(buf, "%lld %s", atoll(mem) / 1024, "MB");
 	} else {
 		sprintf(buf, "%s %s", mem, unit);
 	}
@@ -1152,6 +1159,15 @@ void print_scaled_output(int interval, int count)
 	} while (--count > 0);
 }
 
+static void print_security_flavor(void)
+{
+	char value[64];
+	char *descr;
+
+	get_sysdata("security_flavor", &descr, value);
+	fprintf(stdout, "%-45s: %s\n", descr, value);
+}
+
 static void usage(void)
 {
 	printf("Usage:  lparstat [ options ]\n\tlparstat <interval> [ count ]\n\n"
@@ -1159,6 +1175,7 @@ static void usage(void)
 	       "\t-h, --help		Show this message and exit.\n"
 	       "\t-V, --version	\tDisplay lparstat version information.\n"
 	       "\t-i			Lists details on the LPAR configuration.\n"
+	       "\t-x			Print the security mode settings for the LPAR.\n"
 	       "\t-E			Print SPURR metrics.\n"
 	       "\t-l, --legacy		Print the report in legacy format.\n"
 	       "interval		The interval parameter specifies the amount of time between each report.\n"
@@ -1184,7 +1201,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	while ((c = getopt_long(argc, argv, "iEVhl",
+	while ((c = getopt_long(argc, argv, "iEVhlx",
 				long_opts, &opt_index)) != -1) {
 		switch(c) {
 			case 'i':
@@ -1199,6 +1216,9 @@ int main(int argc, char *argv[])
 			case 'V':
 				printf("lparstat - %s\n", VERSION);
 				return 0;
+			case 'x':
+				o_security = true;
+				break;
 			case 'h':
 				usage();
 				return 0;
@@ -1223,6 +1243,8 @@ int main(int argc, char *argv[])
 
 	if (i_option)
 		print_iflag_data();
+	else if (o_security)
+		print_security_flavor();
 	else if (o_scaled) {
 		print_scaled_output(interval, count);
 		close_cpu_sysfs_fds(threads_in_system);
